'''
CF-1.x checker
implementation largely based on requirements and recommendations from
http://cf-pcmdi.llnl.gov/conformance/requirements-and-recommendations/

non-standard dependencies:
- NetCDF4-python
    - requires numpy
- lxml
- cfunits (https://code.google.com/p/cfunits-python/)
    - requires Unidata UDUNITS2 C Library
'''

__version__ = '1.6.2a'

import os

from netCDF4 import Dataset as ncdf4

from utils import get_cf_version

_packagedir = os.path.dirname(__file__)
_datadir = os.path.join(_packagedir, 'data')

from CFchecker.errors import CFCheckerError, CFAutoDetectError
from CFchecker.cf10 import CF10Tests
from CFchecker.cf11 import CF11Tests
from CFchecker.cf12 import CF12Tests
from CFchecker.cf13 import CF13Tests
from CFchecker.cf14 import CF14Tests
from CFchecker.cf15 import CF15Tests
from CFchecker.cf16 import CF16Tests

available_cf_tests = {'1.0': CF10Tests,
                   '1.1': CF11Tests,
                   '1.2': CF12Tests,
                   '1.3': CF13Tests,
                   '1.4': CF14Tests,
                   '1.5': CF15Tests,
                   '1.6': CF16Tests,
                  }

def get_available_versions():
    return ['auto'] + sorted(available_cf_tests.keys())

def checkfile(filename, cf_version, tests_to_run=None):
    '''
    wrapper for all supported CF-convention versions
    just pass filename to check and desired CF version and
    the correct test will be run automatically
    the MessageLog generated by the test run will then be returned

    if cf_version is set to 'auto', the script tries to determine the appropriate
    convention version from the Conventions attribute

    filename:   file to check
    cf_version: CF convention version to use for check or 'auto'
    tests_to_run: list of testids that should be run or None for all tests
    returns:    MessageLog object containing check results
    '''
    # autodetect convention version
    if cf_version == 'auto':
        ncfile = ncdf4(filename, mode='r')
        try:
            cf_version = get_cf_version(ncfile)
        finally:
            ncfile.close()
        if cf_version is None:
            raise CFAutoDetectError('Auto detection of CF convention version failed because the "Conventions" attribute is missing in the file. '
                                                    'In Order to check the file anyhow, please specify CF conventions version manually. '
                                                    '(NOTE: No CF compliance checks have been performed due to this error.)')

    if cf_version not in available_cf_tests.keys():
        raise CFCheckerError('unknown CF conventions version "%s"' % str(cf_version))
    cftest = available_cf_tests[cf_version](tests_to_run=tests_to_run)
    cftest.run(filename)
    return cftest.get_messagelog()
